

makeCumulative <- function(x)
{
  UseMethod("makeCumulative")
}

makeIncremental <- function(x)
{
  UseMethod("makeIncremental")
}

getLatest <- function(x, drop=F)
{
  UseMethod("getLatest")
}

# ******************************************************************************
# General Functions
# ******************************************************************************


.getIndivDFs <- function(x)
{
  UseMethod(".getIndivDFs")
}


#-------------------------------------------------------------------------------
# Internal: Function to get average development factors
#-------------------------------------------------------------------------------
.getAvDFs <- function(dfs, wghts, include)
{
  # Perform calculations
  .avDFs <- apply(dfs * wghts * include, c(2,3), sum, na.rm=T) /
    apply(wghts * include, c(2,3), sum, na.rm=T)
  .avDFs[is.nan(.avDFs)] <- 1

  .avDFs <- array(.avDFs, dim=c(1, dim(.avDFs)))
 
  .names <- dimnames(dfs)

  .names$origin <- 1
  dimnames(.avDFs) <- .names

  return(.avDFs)
}


#-------------------------------------------------------------------------------
# Function to calculate average development factors
#-------------------------------------------------------------------------------
calculateAvDFs <- function(x, wghts, include, ...)
{
  # Calculate individual Development Factors
  .indivDFs <- .getIndivDFs(x)

  # Determine weights
  if(missing(wghts)) 
    wghts <- as.ArrayTriangle(x)$data
  else {
    # !!!!!!!!!!!!!!!Check appropriate format - LATER !!!!!!!!!!!!!!!!!!!!!!!!!!
  }
  
  # Determine factors to include in calculation
  if(missing(include)) {
    include <- .indivDFs
    include[!is.na(include)] <- 1}
  else {
    # !!!!!!!!!!!!!!!Check appropriate format - LATER !!!!!!!!!!!!!!!!!!!!!!!!!!
  }

  # Determine method
  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 
  # Could have a method variable, which takes last 5 development factors, for 
  # example - LATER
  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! 

 
  # Perform calculation
  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  # Could have get this to project a tail factor - LATER
  # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  .avDFs <- .getAvDFs(.indivDFs, wghts, include)

  .out <- list( call = match.call(),
                indivDFs = .indivDFs,
                weights = wghts,        # Do we want to keep this?
                avDFs = .avDFs
              )
  attr(.out, "class") <- "AvDFs"
  
  # Return result
  return(.out)
}



calculateCLUlts <- function(triangle, avDFs, ...)
{
  # Get latest claims data
  if(triangle$cumulative==F) triangle <- makeCumulative(triangle)
  .latest <- getLatest(triangle)

  # Get average development factors.  Can pass arguments to this function.
  if(missing(avDFs))
    avDFs <- calculateAvDFs(triangle, ...)

  # Convert to ultimate factors
  #!!!!!!!!!!!!!!!!!!!  LATER !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    
  # Match development factors to latest data
  .pos <- data.frame(origin=1, dev=as.numeric(as.vector(.latest$dev)),
    triangle=rep(as.numeric(dimnames(.latest$dev)$triangle),
    each=dim(.latest$dev)[1]))
  .pos$dev <- match(.pos$dev, dimnames(avDFs$avDFs)$dev)
  .pos <- as.matrix(.pos)
  # Then pull through relevant development factors
  .ultDFs <- .latest$latest
  .ultDFs[] <- avDFs$avDFs[.pos]       #  Replace with ultDFs !!!!!!!!!!!!!!!!!!

  # Multiply to get ultimates
  .ult <- .latest$latest * .ultDFs

  return(.ult)
}








# ******************************************************************************


# Sample data for testing
#Mack <- read.csv("C:/Documents and Settings/desilvan/My Documents/Working Party/R/Mack 2.csv", header=F)
Mack <- read.csv("F:/Nigel's Documents/Working Party/R Examples/Mack 2.csv", header=F)
Mack <- as.matrix(Mack)
Mack <- array(Mack, dim=c(dim(Mack),2))

Mack <- ArrayTriangle(Mack, cumulative=F)
